@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000-2004 Anton Altaparmakov
  * Copyright (c) 2002-2009 Szabolcs Szakacsits
- * Copyright (c) 2008-2009 Jean-Pierre Andre
+ * Copyright (c) 2008-2010 Jean-Pierre Andre
  * Copyright (c) 2008      Bernhard Kaindl
  *
  * This program/include file is free software; you can redistribute it and/or
@@ -166,24 +166,22 @@ int ntfs_names_full_collate(const ntfschar *name1, const u32 name1_len,
 	cnt = min(name1_len, name2_len);
 	if (cnt > 0) {
 		if (ic == CASE_SENSITIVE) {
-			do {
-				c1 = le16_to_cpu(*name1);
+			while (--cnt && (*name1 == *name2)) {
 				name1++;
-				c2 = le16_to_cpu(*name2);
 				name2++;
-			} while (--cnt && (c1 == c2));
-			u1 = c1;
-			u2 = c2;
+			}
+			u1 = c1 = le16_to_cpu(*name1);
+			u2 = c2 = le16_to_cpu(*name2);
 			if (u1 < upcase_len)
 				u1 = le16_to_cpu(upcase[u1]);
 			if (u2 < upcase_len)
 				u2 = le16_to_cpu(upcase[u2]);
 			if ((u1 == u2) && cnt)
 				do {
-					u1 = le16_to_cpu(*name1);
 					name1++;
-					u2 = le16_to_cpu(*name2);
+					u1 = le16_to_cpu(*name1);
 					name2++;
+					u2 = le16_to_cpu(*name2);
 					if (u1 < upcase_len)
 						u1 = le16_to_cpu(upcase[u1]);
 					if (u2 < upcase_len)
@@ -389,6 +387,21 @@ void ntfs_name_upcase(ntfschar *name, u32 name_len, const ntfschar *upcase,
 			name[i] = upcase[u];
 }
 
+/**
+ * ntfs_name_locase - Map a Unicode name to its lowercase equivalent
+ */
+void ntfs_name_locase(ntfschar *name, u32 name_len, const ntfschar *locase,
+		const u32 locase_len)
+{
+	u32 i;
+	u16 u;
+
+	if (locase)
+		for (i = 0; i < name_len; i++)
+			if ((u = le16_to_cpu(name[i])) < locase_len)
+				name[i] = locase[u];
+}
+
 /**
  * ntfs_file_value_upcase - Convert a filename to upper case
  * @file_name_attr:
@@ -815,12 +828,15 @@ int ntfs_ucstombs(const ntfschar *ins, const int ins_len, char **outs,
 		int outs_len)
 {
 	char *mbs;
+	int mbs_len;
+#ifdef MB_CUR_MAX
 	wchar_t wc;
-	int i, o, mbs_len;
+	int i, o;
 	int cnt = 0;
 #ifdef HAVE_MBSINIT
 	mbstate_t mbstate;
 #endif
+#endif /* MB_CUR_MAX */
 
 	if (!ins || !outs) {
 		errno = EINVAL;
@@ -834,6 +850,7 @@ int ntfs_ucstombs(const ntfschar *ins, const int ins_len, char **outs,
 	}
 	if (use_utf8)
 		return ntfs_utf16_to_utf8(ins, ins_len, outs, outs_len);
+#ifdef MB_CUR_MAX
 	if (!mbs) {
 		mbs_len = (ins_len + 1) * MB_CUR_MAX;
 		mbs = ntfs_malloc(mbs_len);
@@ -899,6 +916,9 @@ err_out:
 		free(mbs);
 		errno = eo;
 	}
+#else /* MB_CUR_MAX */
+	errno = EILSEQ;
+#endif /* MB_CUR_MAX */
 	return -1;
 }
 
@@ -927,6 +947,7 @@ err_out:
  */
 int ntfs_mbstoucs(const char *ins, ntfschar **outs)
 {
+#ifdef MB_CUR_MAX
 	ntfschar *ucs;
 	const char *s;
 	wchar_t wc;
@@ -934,6 +955,7 @@ int ntfs_mbstoucs(const char *ins, ntfschar **outs)
 #ifdef HAVE_MBSINIT
 	mbstate_t mbstate;
 #endif
+#endif /* MB_CUR_MAX */
 
 	if (!ins || !outs) {
 		errno = EINVAL;
@@ -943,6 +965,7 @@ int ntfs_mbstoucs(const char *ins, ntfschar **outs)
 	if (use_utf8)
 		return ntfs_utf8_to_utf16(ins, outs);
 
+#ifdef MB_CUR_MAX
 	/* Determine the size of the multi-byte string in bytes. */
 	ins_size = strlen(ins);
 	/* Determine the length of the multi-byte string. */
@@ -1032,9 +1055,67 @@ int ntfs_mbstoucs(const char *ins, ntfschar **outs)
 	return o;
 err_out:
 	free(ucs);
+#else /* MB_CUR_MAX */
+	errno = EILSEQ;
+#endif /* MB_CUR_MAX */
 	return -1;
 }
 
+/*
+ *		Turn a UTF8 name uppercase
+ *
+ *	Returns an allocated uppercase name which has to be freed by caller
+ *	or NULL if there is an error (described by errno)
+ */
+
+char *ntfs_uppercase_mbs(const char *low,
+			const ntfschar *upcase, u32 upcase_size)
+{
+	int size;
+	char *upp;
+	u32 wc;
+	int n;
+	const char *s;
+	char *t;
+
+	size = strlen(low);
+	upp = (char*)ntfs_malloc(3*size + 1);
+	if (upp) {
+		s = low;
+		t = upp;
+		do {
+			n = utf8_to_unicode(&wc, s);
+			if (n > 0) {
+				if (wc < upcase_size)
+					wc = le16_to_cpu(upcase[wc]);
+				if (wc < 0x80)
+					*t++ = wc;
+				else if (wc < 0x800) {
+					*t++ = (0xc0 | ((wc >> 6) & 0x3f));
+					*t++ = 0x80 | (wc & 0x3f);
+				} else if (wc < 0x10000) {
+					*t++ = 0xe0 | (wc >> 12);
+					*t++ = 0x80 | ((wc >> 6) & 0x3f);
+					*t++ = 0x80 | (wc & 0x3f);
+				} else {
+					*t++ = 0xf0 | ((wc >> 18) & 7);
+					*t++ = 0x80 | ((wc >> 12) & 63);
+					*t++ = 0x80 | ((wc >> 6) & 0x3f);
+					*t++ = 0x80 | (wc & 0x3f);
+				}
+			s += n;
+			}
+		} while (n > 0);
+		if (n < 0) {
+			free(upp);
+			upp = (char*)NULL;
+			errno = EILSEQ;
+		}
+		*t = 0;
+	}
+	return (upp);
+}
+
 /**
  * ntfs_upcase_table_build - build the default upcase table for NTFS
  * @uc:		destination buffer where to store the built table
@@ -1106,6 +1187,38 @@ void ntfs_upcase_table_build(ntfschar *uc, u32 uc_len)
 	}
 }
 
+/*
+ *		Build a table for converting to lower case
+ *
+ *	This is only meaningful when there is a single lower case
+ *	character leading to an upper case one, and currently the
+ *	only exception is the greek letter sigma which has a single
+ *	upper case glyph (code U+03A3), but two lower case glyphs
+ *	(code U+03C3 and U+03C2, the latter to be used at the end
+ *	of a word). In the following implementation the upper case
+ *	sigma will be lowercased as U+03C3.
+ */
+
+ntfschar *ntfs_locase_table_build(const ntfschar *uc, u32 uc_cnt)
+{
+	ntfschar *lc;
+	u32 upp;
+	u32 i;
+
+	lc = (ntfschar*)ntfs_malloc(uc_cnt*sizeof(ntfschar));
+	if (lc) {
+		for (i=0; i<uc_cnt; i++)
+			lc[i] = cpu_to_le16(i);
+		for (i=0; i<uc_cnt; i++) {
+			upp = le16_to_cpu(uc[i]);
+			if ((upp != i) && (upp < uc_cnt))
+				lc[upp] = cpu_to_le16(i);
+		}
+	} else
+		ntfs_log_error("Could not build the locase table\n");
+	return (lc);
+}
+
 /**
  * ntfs_str2ucs - convert a string to a valid NTFS file name
  * @s:		input string
@@ -1177,7 +1290,9 @@ BOOL ntfs_forbidden_chars(const ntfschar *name, int len)
 			| (1L << ('>' - 0x20))
 			| (1L << ('?' - 0x20));
 
-	forbidden = (len == 0) || (le16_to_cpu(name[len-1]) == ' ');
+	forbidden = (len == 0)
+			|| (le16_to_cpu(name[len-1]) == ' ')
+			|| (le16_to_cpu(name[len-1]) == '.');
 	for (i=0; i<len; i++) {
 		ch = le16_to_cpu(name[i]);
 		if ((ch < 0x20)
