@@ -202,6 +202,7 @@ static int __ntfs_volume_release(ntfs_volume *v)
 	ntfs_free_lru_caches(v);
 	free(v->vol_name);
 	free(v->upcase);
+	if (v->locase) free(v->locase);
 	free(v->attrdef);
 	free(v);
 
@@ -488,7 +489,14 @@ ntfs_volume *ntfs_volume_startup(struct ntfs_device *dev, unsigned long flags)
 	
 	ntfs_upcase_table_build(vol->upcase,
 			vol->upcase_len * sizeof(ntfschar));
+	/* Default with no locase table and case sensitive file names */
+	vol->locase = (ntfschar*)NULL;
+	NVolSetCaseSensitive(vol);
 	
+		/* by default, all files are shown and not marked hidden */
+	NVolSetShowSysFiles(vol);
+	NVolSetShowHidFiles(vol);
+	NVolClearHideDotFiles(vol);
 	if (flags & MS_RDONLY)
 		NVolSetReadOnly(vol);
 	
@@ -860,6 +868,7 @@ ntfs_volume *ntfs_device_mount(struct ntfs_device *dev, unsigned long flags)
 	VOLUME_INFORMATION *vinf;
 	ntfschar *vname;
 	int i, j, eo;
+	unsigned int k;
 	u32 u;
 
 	vol = ntfs_volume_startup(dev, flags);
@@ -1017,6 +1026,17 @@ ntfs_volume *ntfs_device_mount(struct ntfs_device *dev, unsigned long flags)
 		ntfs_log_perror("Failed to close $UpCase");
 		goto error_exit;
 	}
+	/* Consistency check of $UpCase, restricted to plain ASCII chars */
+	k = 0x20;
+	while ((k < vol->upcase_len)
+	    && (k < 0x7f)
+	    && (le16_to_cpu(vol->upcase[k])
+			== ((k < 'a') || (k > 'z') ? k : k + 'A' - 'a')))
+		k++;
+	if (k < 0x7f) {
+		ntfs_log_error("Corrupted file $UpCase\n");
+		goto io_error_exit;
+	}
 
 	/*
 	 * Now load $Volume and set the version information and flags in the
@@ -1195,6 +1215,58 @@ error_exit:
 	return NULL;
 }
 
+/*
+ *		Set appropriate flags for showing NTFS metafiles
+ *	or files marked as hidden.
+ *	Not set in ntfs_mount() to avoid breaking existing tools.
+ */
+
+int ntfs_set_shown_files(ntfs_volume *vol,
+			BOOL show_sys_files, BOOL show_hid_files,
+			BOOL hide_dot_files)
+{
+	int res;
+
+	res = -1;
+	if (vol) {
+		NVolClearShowSysFiles(vol);
+		NVolClearShowHidFiles(vol);
+		NVolClearHideDotFiles(vol);
+		if (show_sys_files)
+			NVolSetShowSysFiles(vol);
+		if (show_hid_files)
+			NVolSetShowHidFiles(vol);
+		if (hide_dot_files)
+			NVolSetHideDotFiles(vol);
+		res = 0;
+	}
+	if (res)
+		ntfs_log_error("Failed to set file visibility\n");
+	return (res);
+}
+
+/*
+ *		Set ignore case mode
+ */
+
+int ntfs_set_ignore_case(ntfs_volume *vol)
+{
+	int res;
+
+	res = -1;
+	if (vol && vol->upcase) {
+		vol->locase = ntfs_locase_table_build(vol->upcase,
+					vol->upcase_len);
+		if (vol->locase) {
+			NVolClearCaseSensitive(vol);
+			res = 0;
+		}
+	}
+	if (res)
+		ntfs_log_error("Failed to set ignore_case mode\n");
+	return (res);
+}
+
 /**
  * ntfs_mount - open ntfs volume
  * @name:	name of device/file to open
